@@ -1,10 +1,11 @@
 
 package Class::Trait::Reflection;
-$VERSION  = '0.01';
 
 use strict;
 use warnings;
 
+our $VERSION  = '0.02';
+
 use Class::Trait;
 
 use Data::Dumper ();
@@ -228,11 +229,13 @@ B<Class::Trait>, B<Class::Trait::Config>
 
 =head1 AUTHOR
 
-Stevan Little E<lt>stevan_little@yahoo.comE<gt>
+Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2004 by Stevan Little
+Copyright 2004 by Infinity Interactive, Inc.
+
+L<http://www.iinteractive.com> 
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
